// -*- C++ -*-
//===----------------------------------------------------------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

export namespace std {

  // [any.bad.any.cast], class bad_any_cast
  using std::bad_any_cast;

  // [any.class], class any
  using std::any;

  // [any.nonmembers], non-member functions
  using std::any_cast;
  using std::make_any;
  using std::swap;

} // namespace std
